from ast import Str
from logging import root
from pydantic import BaseModel, validator, EmailStr
from datetime import date
from typing import Optional, List, Union
from model.common.employerbase import EmployerBase
from model.common.jobofferbase import JobofferBase
from model.common.contact import ContactBase
from pydantic.class_validators import root_validator
from model.common.utils import checkRow
from model.common.utils import makeList

# employer classes
class General(EmployerBase):
    company_intro: str
    business_intro: str
    recruit_email: EmailStr
    cra_number: str
    ft_employee_number: int
    pt_employee_number: int
    establish_date: date
    industry: str
    registration_number: str


class Contact(ContactBase):
    position: Optional[str]

    @root_validator
    def checkCompletion(cls, values):
        all_fields = ["last_name", "first_name", "phone", "email", "position"]
        required_fields = ["last_name", "first_name", "phone", "email", "position"]
        checkRow(values, all_fields, required_fields)
        return values


# personal classes
class JobOffer(JobofferBase):
    offer_date: date
    duties: list
    specific_edu_requirement: Optional[str]
    skill_experience_requirement: Optional[str]
    other_requirements: list

    _str2bool_duties = validator("duties", allow_reuse=True, pre=True)(makeList)
    _str2bool_other_requirements = validator(
        "other_requirements", allow_reuse=True, pre=True
    )(makeList)

    @property
    def date_of_offer(self):
        return self.offer_date.strftime("%b %d, %Y")

    @property
    def requirements(self):
        return [
            r
            for r in [
                self.specific_edu_requirement,
                self.skill_experience_requirement,
                *self.other_requirements,
            ]
            if r is not None
        ]


class PersonalAssess(BaseModel):
    work_experience_brief: str
    education_brief: str
    competency_brief: str
    language_brief: Optional[str]
    performance_remark: Optional[str]

    @property
    def why_qualified(self):
        qualifications = [
            self.work_experience_brief,
            self.education_brief,
            self.competency_brief,
            self.language_brief,
            self.performance_remark,
        ]
        return [q for q in qualifications if q is not None]


# lmia classes
class LmiaCase(BaseModel):
    purpose_of_lmia: str
    stream_of_lmia: str
    number_of_tfw: int
    area_median_wage: float
    provincial_median_wage: float
    duration_number: int
    duration_unit: str
    duration_reason: str
    has_attestation: bool

    @property
    def purpose_say(self):
        return self.purpose_of_lmia

    @property
    def stream_say(self):
        return self.stream_of_lmia

    @property
    def duration_say(self):
        return (
            str(self.duration_number) + " " + self.duration_unit + "s"
            if self.duration_number > 1
            else str(self.duration_number) + " " + self.duration_unit
        )

    @property
    def stars(self):
        return 2 if self.stream_of_lmia.lower() in ["low wage stream", "lws"] else 4


class Finance(BaseModel):
    year: int
    revenue: float
    net_income: float
    retained_earning: float

    @property
    def formatted_revenue(self):
        return "{:,.0f}".format(self.revenue)

    @property
    def formatted_net_income(self):
        return "{:,.0f}".format(self.net_income)

    @property
    def formatted_retained_earning(self):
        return "{:,.0f}".format(self.retained_earning)


class Lmi(BaseModel):
    brief_benefit: str
    job_creation_benefit: Optional[str]
    skill_transfer_benefit: Optional[str]
    fill_shortage_benefit: Optional[str]
    other_benefit: Optional[str]

    @property
    def benefits(self):
        return [
            b
            for b in [
                self.job_creation_benefit,
                self.skill_transfer_benefit,
                self.fill_shortage_benefit,
                self.other_benefit,
            ]
            if b is not None
        ]


# RCIC
class Rcic(BaseModel):
    first_name: str
    last_name: str
    sex: str
    rcic_number: str
    email: EmailStr
    company: str
    rcic_rate: str
    how_client_know_rcic: str

    @property
    def name(self):
        return self.first_name + " " + self.last_name


class Emp5593(BaseModel):
    pass


class Emp5626(BaseModel):
    is_seasonal: bool
    start_month: Optional[str]
    end_month: Optional[str]
    last_canadian_number: Optional[int]
    last_tfw_number: Optional[int]
    current_canadian_number: int
    current_tfw_number: int
    tp_waivable: bool
    waive_creteria: Optional[str]
    has_finished_tp: bool
    finished_tp_result: Optional[str]
    activity1_title: Optional[str]
    activity1_decription: Optional[str]
    activity1_outcome: Optional[str]
    activity1_commnet: Optional[str]
    activity2_title: Optional[str]
    activity2_decription: Optional[str]
    activity2_outcome: Optional[str]
    activity2_commnet: Optional[str]
    activity3_title: Optional[str]
    activity3_decription: Optional[str]
    activity3_outcome: Optional[str]
    activity3_commnet: Optional[str]
    activity4_title: Optional[str]
    activity4_decription: Optional[str]
    activity4_outcome: Optional[str]
    activity4_commnet: Optional[str]
    activity5_title: Optional[str]
    activity5_decription: Optional[str]
    activity5_outcome: Optional[str]
    activity5_commnet: Optional[str]


class Emp5627(BaseModel):
    provide_accommodation: bool
    description: Optional[str]
    rent_unit: str
    rent_amount: float
    accommodation_type: str
    explain: Optional[str]
    bedrooms: int
    people: int
    bathrooms: int
    other: Optional[str]



# from context import BASEDIR
from source.excel import Excel
from model.common.modelloader import ExcelMaker, valid_programs
import argparse
from functools import reduce
from termcolor import colored
import json


def main():
    parser = argparse.ArgumentParser(
        description="For munipulate excel and get data source..."
    )

    parser.add_argument("-e", "--excel", help="Input excel name")
    parser.add_argument(
        "-a",
        "--add",
        help="add up excels(can be more than 2), and output to another excel",
        nargs="+",
    )
    parser.add_argument(
        "-s",
        "--sub",
        help="sub excels(can be more than 2), and output to another excel",
        nargs="+",
    )
    parser.add_argument(
        "-c",
        "--copy",
        help="copy later excels' common contents to first excel(can be more than 2), and output to another excel",
        nargs="+",
    )
    parser.add_argument(
        "-t",
        "--to",
        help="Output file name, if not input, the add, sub,or copy's first filename will be used as default",
    )
    parser.add_argument(
        "-po",
        "--protection_off",
        help="Protection off. default is On",
        action="store_true",
    )
    parser.add_argument("-j", "--json", help="need json output", action="store_true")
    parser.add_argument("-d", "--dict", help="need dict output", action="store_true")

    # Deprecated. Now modelmakecheck app will handle all excel generation based on model
    # parser.add_argument(
    #     "-p",
    #     "--program",
    #     help="input program code. 5257 for trv, 1294 or 1295 for sp or wp outside of Canada. 5708/5709/5710 for vr/sp/wp in Canada ",
    # )

    args = parser.parse_args()
    protection = False if args.protection_off else True

    if args.add:
        if len(args.add) < 2:
            raise ValueError("Must more than two excel files after -a ")
        output_fn = args.to or args.add[0]
        try:
            excel_objs = list(map(lambda x: Excel(x), args.add))
            e = reduce(lambda a, b: a + b, excel_objs)
            e.makeExcel(output_fn, protection=protection)
            print(colored(f"{output_fn} saved", "green"))
            if args.json:
                print(e.json)
            if args.dict:
                print(e.dict)
        except Exception as e:
            print(colored(f"{str(e)}", "red"))

        return

    if args.sub:
        if len(args.sub) < 2:
            raise ValueError("Must more than two excel files after -a ")
        output_fn = args.to or args.sub[0]
        try:
            excel_objs = list(map(lambda x: Excel(x), args.sub))
            e = reduce(lambda a, b: a - b, excel_objs)
            e.makeExcel(output_fn, protection=protection)
            print(colored(f"{output_fn} saved", "green"))
            if args.json:
                print(e.json)
            if args.dict:
                print(e.dict)

        except Exception as e:
            print(colored(f"{str(e)}", "red"))

        return

    if args.copy:
        target_name = (
            args.to or args.copy[0]
        )  # if without -t filename, use first file as the target file name
        if len(args.copy) < 2:
            raise ValueError("Must  more than two excel files after -c ")
        try:
            excel_objs = list(map(lambda x: Excel(x), args.copy))
            e = reduce(lambda a, b: a.copy(b), excel_objs)
            e.makeExcel(target_name, protection=protection)
            print(colored(f"{target_name} saved", "green"))
            if args.json:
                print(e.json)
            if args.dict:
                print(e.dict)

        except Exception as e:
            print(colored(f"{str(e)}", "red"))

        return

    # if args.program and args.to:
    #     if args.program not in valid_programs:
    #         print(f"Your input program code {args.program} is not valid. ")
    #         return
    #     excel_file = ExcelMaker(args.program)
    #     excel_file.makeExcelBasedOnModel(args.to)

    if args.excel and args.to and args.json:
        excel = Excel(args.excel)
        with open(args.to, "w") as fp:
            json.dump(excel.dict, fp, indent=3, default=str)
        print(colored(f"{args.to} is saved", "green"))
        return

    if args.excel and args.json:
        excel = Excel(args.excel)
        print(excel.json)


if __name__ == "__main__":
    main()



from termcolor import colored
from pydantic import BaseModel, EmailStr, root_validator
from functools import reduce
from typing import List, Optional
from model.common.address import Address
from model.common.mixins import DatePeriod
from model.common.phone import Phone
from model.common.commonmodel import CommonModel
from model.common.educationbase import EducationBase
from model.common.utils import checkRow
from utils.utils import checkContinuity
from datetime import date
from pprint import pprint


class Family(BaseModel):
    relationship: Optional[str]
    last_name: Optional[str]
    first_name: Optional[str]
    native_last_name: Optional[str]
    native_first_name: Optional[str]
    date_of_birth: Optional[date]
    date_of_death: Optional[date]
    place_of_birth: Optional[str]
    birth_country: Optional[str]
    marital_status: Optional[str]
    email: Optional[EmailStr]
    address: Optional[str]

    @root_validator
    def checkCompletion(cls, values):
        all_fields = [
            "relationship",
            "last_name",
            "first_name",
            "native_last_name",
            "native_first_name",
            "date_of_birth",
            "date_of_death",
            "place_of_birth",
            "birth_country",
            "marital_status",
            "email",
            "address",
        ]
        required_fields = [
            "relationship",
            "last_name",
            "first_name",
            "native_last_name",
            "native_first_name",
            "date_of_birth",
            "place_of_birth",
            "birth_country",
            "marital_status",
            "address",
        ]
        checkRow(values, all_fields, required_fields)
        return values


class Personal(BaseModel):
    last_name: str
    first_name: str
    native_last_name: str
    native_first_name: str
    dob: date


class PRBackground(BaseModel):
    q1: bool
    q2: bool
    q3: bool
    q4: bool
    q5: bool
    q6: bool
    q7: bool
    q8: bool
    q9: bool
    q10: bool
    q11: bool
    details: Optional[str]


class Education(EducationBase):
    city: Optional[str]
    country: Optional[str]


class History(DatePeriod):
    activity: str
    city_and_country: str
    status: str
    name_of_company_or_school: str

    @property
    def __str__(self) -> str:
        return "table-history"


class Member(DatePeriod):
    organization_name: str
    organization_type: str
    position: str
    city: str
    country: str


class Government(DatePeriod):
    country: str
    department: str
    position: str


class Military(DatePeriod):
    country: str
    service_detail: str
    rank: str
    combat_detail: Optional[str]
    reason_for_end: Optional[str]


class AddressHistory(DatePeriod):
    street_and_number: str
    city: str
    province: str
    country: str
    post_code: str

    @property
    def __str__(self) -> str:
        return "table-addresshistory"


class M5669Model(CommonModel):
    personal: Personal
    family: List[Family]
    prbackground: PRBackground
    education: List[Education]
    history: List[History]
    member: List[Member]
    government: List[Government]
    military: List[Military]
    addresshistory: List[AddressHistory]

    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self, excels=None, output_excel_file=None, check=False):
        if output_excel_file:
            excels = self.getExcels(["excel/pr.xlsx", "excel/pa.xlsx"])
        else:
            if excels is None and len(excels) == 0:
                raise ValueError(
                    "You must input excel file list as source data for validation"
                )

        super().__init__(excels, output_excel_file, globals())

        if check:
            [self.check(item) for item in [self.history, self.addresshistory]]

    def check(self, items: List[object]):
        results = []
        # construct list suitable for checkContinuity
        for item in items:
            results.append(list(item.__dict__.values()))

        # check
        continued, sorted_list, msg = checkContinuity(results)

        if not continued:
            print(
                colored(
                    f"There are {len(msg)} error(s) in sheet {items[0].__str__}", "red"
                )
            )
            [print(index, "\t", m) for index, m in enumerate(msg)]



import argparse, json
from turtle import color
from termcolor import colored
from tabulate import tabulate
from os.path import exists
from model.common.wordmaker import WordMaker
import os

# Get project's home directory,
BASEDIR = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
# All data directory
DATADIR = os.path.abspath(os.path.join(BASEDIR, "data"))


def getArgs():
    parser = argparse.ArgumentParser(
        description="used for making excel based on model or checking excel data according to the model"
    )
    parser.add_argument("model", help="input program code: 0008 5669 5562 5406 ")
    parser.add_argument("-m", "--make", help="input excel file name for output")
    parser.add_argument(
        "-e",
        "--excel",
        help="input excel file name as data source for generating docx file",
    )
    parser.add_argument("-w", "--word", help="input word file name for output")
    parser.add_argument(
        "-r",
        "--rep_company",
        help="input rep's company short name for generating some customized docs",
    )
    parser.add_argument("-c", "--check", help="input excel file name to check")
    parser.add_argument("-j", "--json", help="output json data", action="store_true")

    args = parser.parse_args()
    args.rep_company = args.rep_company or "noah"
    return args


def getModels(args):
    models = {
        # Experience
        "exp-ec": {
            "path": "model.experience.ec",
            "class_list": ["EmploymentModel"],
            "remark": "Experience module for Employment Certificate model",
        },
        "exp-rs": {
            "path": "model.experience.resume",
            "class_list": ["ResumeModel"],
            "remark": "Experience module for Resume model",
        },
        # Recruitment
        "recruit-ja": {
            "path": "model.recruit.jobad",
            "class_list": ["JobadModel"],
            "remark": "Recruit module for Job Advertisement model",
        },
        "recruit-jo": {
            "path": "model.recruit.joboffer",
            "class_list": ["JobofferModel"],
            "remark": "Recruitment module for Job Offer model",
        },
        "recruit-rs": {
            "path": "model.recruit.recruitmentsummary",
            "class_list": ["RecruitmnetSummaryModel"],
            "remark": "Recruitment module for Recruitment Summary model",
        },
        # LMIA
        "lmia-ert": {
            "path": "model.lmia.employertraining",
            "class_list": ["EmployerTrainingModel"],
            "remark": "LMIA module for Employer Training model",
        },
        "lmia-sl": {
            "path": "model.lmia.submissionletter",
            "class_list": ["SubmissionLetterModel"],
            "remark": "LMIA module for Submission Letter model",
        },
        "lmia-st1": {
            "path": "model.lmia.stage1",
            "class_list": ["LmiaAssess"],
            "remark": "LMIA module for stage 1 model (assessment)",
        },
        "lmia-st2": {
            "path": "model.lmia.stage2",
            "class_list": ["LmiaRecruitment"],
            "remark": "LMIA module for stage 2 model (recruitment)",
        },
        "lmia-st3": {
            "path": "model.lmia.stage3",
            "class_list": ["LmiaApplication"],
            "remark": "LMIA module for for stage 3 model (application)",
        },
        "lmia-rcic": {
            "path": "model.lmia.rcic",
            "class_list": ["LmiaRcic"],
            "remark": "LMIA module for RCIC model (Plannning)",
        },
        # BCPNP
        "bcpnp-ert": {
            "path": "model.bcpnp.employertraining",
            "class_list": ["EmployerTrainingModel"],
            "remark": "BCPNP module for Employer Training model",
        },
        "bcpnp-eet": {
            "path": "model.bcpnp.employeetraining",
            "class_list": ["EmployeeTrainingModel"],
            "remark": "BCPNP module for Employee Training model",
        },
        "bcpnp-jd": {
            "path": "model.bcpnp.jobdescription",
            "class_list": ["JobDescriptionModel"],
            "remark": "BCPNP module for Job Description model",
        },
        "bcpnp-jof": {
            "path": "model.bcpnp.jobofferform",
            "class_list": ["JobOfferFormModel"],
            "remark": "BCPNP module for Job Offer Form model",
        },
        "bcpnp-rl": {
            "path": "model.bcpnp.recommendationletter",
            "class_list": ["RecommendationLetterModel"],
            "remark": "BCPNP module for Recommendation Letter model",
        },
        "bcpnp-rpf": {
            "path": "model.bcpnp.repform",
            "class_list": ["RepFormModel"],
            "remark": "BCPNP module for Representative Form model",
        },
        "bcpnp-reg": {
            "path": "webform.bcpnp.bcpnpmodel_reg",
            "class_list": ["BcpnpModelReg"],
            "remark": "BCPNP module for BCPNP Registration model",
        },
        "bcpnp-reg-ee": {
            "path": "webform.bcpnp.bcpnpmodel_reg",
            "class_list": ["BcpnpEEModelReg"],
            "remark": "BCPNP module for BCPNP Registration model(EE)",
        },
        "bcpnp-app": {
            "path": "webform.bcpnp.bcpnpmodel_app",
            "class_list": ["BcpnpModelApp"],
            "remark": "BCPNP module for BCPNP Application model",
        },
        "bcpnp-app-ee": {
            "path": "webform.bcpnp.bcpnpmodel_app",
            "class_list": ["BcpnpEEModelApp"],
            "remark": "BCPNP module for BCPNP application (EE) model",
        },
        # PR
        "0008": {
            "path": "model.pr.m0008",
            "class_list": ["M0008Model"],
            "remark": "PR module for form 0008 model",
        },
        "5406": {
            "path": "model.pr.m5406",
            "class_list": ["M5406Model"],
            "remark": "PR module for form 5406 model",
        },
        "5562": {
            "path": "model.pr.m5562",
            "class_list": ["M5562Model"],
            "remark": "PR module for form 5562 model",
        },
        "5669": {
            "path": "model.pr.m5669",
            "class_list": ["M5669Model"],
            "remark": "PR module for form 5669 model",
        },
        "pr": {
            "path": "webform.prportal.prmodel",
            "class_list": ["PrModel"],
            "remark": "PR module for all PR model",
        },
        # TR
        "0104": {
            "path": "model.tr.m0104",
            "class_list": ["M0104Model"],
            "remark": "TR module for form 0104 model",
        },
        "1294": {
            "path": "model.tr.m1294",
            "class_list": ["M1294Model"],
            "remark": "TR module for form 1294 model",
        },
        "1295": {
            "path": "model.tr.m1295",
            "class_list": ["M1295Model"],
            "remark": "TR module for form 1295 model",
        },
        "5257": {
            "path": "model.tr.m5257",
            "class_list": ["M5257Model"],
            "remark": "TR module for form 5257 model",
        },
        "5708": {
            "path": "model.tr.m5708",
            "class_list": ["M5708Model"],
            "docx_template": os.path.join(
                DATADIR, "word", f"{args.rep_company}-5708-sl.docx"
            ),
            "remark": "TR module for form 5708 model",
        },
        "5709": {
            "path": "model.tr.m5709",
            "class_list": ["M5709Model"],
            "remark": "TR module for form 5709 model",
        },
        "5710": {
            "path": "model.tr.m5710",
            "class_list": ["M5710Model"],
            "remark": "TR module for form 5710 model",
        },
    }

    return models


def makeExcel(args, the_model):
    file_exists = exists(args.make)
    if file_exists:
        overwrite = input(colored(f"{args.make} is existed, overwrite?(Y/N)"))
        if overwrite and overwrite.upper()[0] == "N":
            return
    # create excel
    the_model(output_excel_file=args.make)
    print(colored(f"{args.make} has been created", "green"))


def makeWord(args, model, the_model):
    word_file_exists = exists(args.word)
    excel_file_exists = exists(args.excel)
    if word_file_exists:
        overwrite = input(colored(f"{args.word} is existed, overwrite?(Y/N)"))
        if overwrite and overwrite.upper()[0] == "N":
            return
    if not excel_file_exists:
        print(colored(f"{args.excel} is not existed."))
        return

    template_docx = model.get("docx_template")
    if not template_docx:
        print(
            colored(
                f"There is no template existing in model {model.get('class_list')[0]}",
                "red",
            )
        )
        return
    if not exists(template_docx):
        print(
            colored(
                f"{template_docx} is not existed. Please check your template base",
                "red",
            )
        )
        return

    model = the_model(excels=[args.excel])
    wm = WordMaker(template_docx, model.getAllDict, args.word)
    wm.make()


def checkModel(args, the_model):
    model_obj = the_model(excels=[args.check])
    if model_obj.dict() != {}:
        print(
            json.dumps(model_obj.dict(), indent=3, default=str)
        ) if args.json else print(
            colored(f"The model has been checked and everything seems good...", "green")
        )


def getHelp(valid_models):
    contents = [["Model", "Description", "Word Template"]]
    for model, value in valid_models.items():
        word_template = value.get("docx_template") or "Not available"
        contents.append([model, value["remark"], os.path.basename(word_template)])

    print(tabulate(contents, tablefmt="fancy_grid"))
    return


def main():
    args = getArgs()
    valid_models = getModels(args)  # some variables depending on args

    if not args.model:
        print(colored("You must input model!", "red"))
        exit(1)

    if args.model.upper() == "HELP":
        getHelp(valid_models)
        return

    if not valid_models.get(args.model):
        print(colored(f"Model ({args.model}) is not existed. Please check.", "red"))
        exit(1)

    # get model, and its instance
    model = valid_models.get(args.model)
    class_list = model["class_list"]
    the_class = __import__(model["path"], fromlist=class_list)
    the_model = getattr(the_class, class_list[0])

    # Make excel based on model
    if args.make:
        makeExcel(args, the_model)

    # Make document based on model and source excel data
    if args.excel and args.word:
        makeWord(args, model, the_model)

    # Check model, output error list or excel file, and generate json output if model is correct and required
    if args.check:
        checkModel(args, the_model)


if __name__ == "__main__":
    main()



from typing import List, Optional, Union
from model.common.address import Address
from model.common.educationbase import EducationBase
from model.common.phone import Phone
from model.common.commonmodel import CommonModel
from model.common.id import ID
from datetime import date
from pydantic import BaseModel, EmailStr, root_validator, validator
from model.common.utils import normalize
from model.common.mixins import DatePeriod
from termcolor import colored


class PersonId(ID):
    pass


class PRCase(BaseModel):
    imm_program: str
    imm_category: str
    imm_under: Optional[str]
    communication_language: str
    interview_language: str
    need_translator: bool
    intended_province: str
    intended_city: str


class COR(DatePeriod):
    start_date: date
    end_date: Union[date, None]
    country: str
    status: str

    def is_current(self):
        return not self.end_date


class CORs(object):
    def __init__(self, cors: List[COR]):
        self.cors = cors
        self.checkFirstEndDate()

    @property
    def current(self):
        if len(self.cors) >= 1:
            return self.cors[0]
        else:
            raise ValueError(
                colored(
                    f"table-cor must have values, and the first line must be current residence country.",
                    "red",
                )
            )

    # In table cor, the first line is defined as current residence, so the end date must bigger than today
    def checkFirstEndDate(self):
        if (
            len(self.cors) > 0
            and self.cors[0].end_date
            and self.cors[0].end_date < date.today()
        ):
            raise ValueError(
                colored(
                    f"In tabel-cor, the current residence end date {self.cors[0].end_date} is earlier than today",
                    "red",
                )
            )


class Marriage(BaseModel):
    marital_status: str
    married_date: Optional[date]
    sp_last_name: Optional[str]
    sp_first_name: Optional[str]
    previous_married: bool
    pre_sp_last_name: Optional[str]
    pre_sp_first_name: Optional[str]
    pre_sp_dob: Optional[date]
    pre_relationship_type: Optional[str]
    pre_start_date: Optional[date]
    pre_end_date: Optional[date]

    @validator("pre_end_date")
    def endDateBigger(cls, pre_end_date, values):
        pre_start_date = values.get("pre_start_date")
        if not pre_start_date:
            return pre_end_date
        the_date = pre_end_date or date.today()
        if (the_date - pre_start_date).days <= 0:
            raise ValueError(
                f"Expiry date {the_date} is earlier than issue date {pre_start_date} for the ID"
                if pre_end_date
                else f"{pre_start_date} is later than today"
            )
        return pre_end_date

    @root_validator
    def checkPreviousMarriage(cls, values):
        pre_marriaged = values["previous_married"]
        pre_sp_last_name = values["pre_sp_last_name"]
        pre_sp_first_name = values["pre_sp_first_name"]
        pre_sp_dob = values["pre_sp_dob"]
        pre_start_date = values["pre_start_date"]
        pre_end_date = values["pre_end_date"]
        pre_relationship_type = values["pre_relationship_type"]

        if pre_marriaged and not all(
            [
                pre_sp_first_name,
                pre_sp_last_name,
                pre_sp_dob,
                pre_start_date,
                pre_end_date,
                pre_relationship_type,
            ]
        ):
            raise ValueError(
                f"Since previous married is true, but you did not answer all the questions for previous spouse: first/last name, dob,relationship type, start/end date"
            )
        return values

    @root_validator
    def checkCurrentMarriage(cls, values):
        marital_status = values["marital_status"]
        married_date = values["married_date"]
        sp_last_name = values["sp_last_name"]
        sp_first_name = values["sp_first_name"]

        if marital_status in ["Common-Law", "Married"] and not all(
            [married_date, sp_last_name, sp_first_name]
        ):
            raise ValueError(
                f"Marital stauts is {marital_status}, but you did not answer all the questions for spouse: first/last name, or marriage date"
            )
        return values


class Status(BaseModel):
    last_entry_date: Optional[date]
    last_entry_place: Optional[str]


class Personal(BaseModel):
    last_name: str
    first_name: str
    used_last_name: Optional[str]
    used_first_name: Optional[str]
    sex: str
    height: int
    eye_color: str
    dob: date
    country_of_birth: str
    place_of_birth: str
    uci: Optional[str]
    citizen: str
    citizen2: Optional[str]
    native_language: str
    english_french: str
    which_one_better: Optional[str]
    language_test: bool
    current_occupation: str
    intended_occupation: Optional[str]
    email: EmailStr
    relationship_to_pa: Optional[str]
    accompany_to_canada: Optional[bool]
    dependant_type: Optional[str]

    _normalize_used_first_name = validator(
        "used_first_name", allow_reuse=True, check_fields=False
    )(normalize)
    _normalize_used_last_name = validator(
        "used_last_name", allow_reuse=True, check_fields=False
    )(normalize)

    @root_validator
    def checkAnswers(cls, values):
        questions = ["english_french"]
        explanations = ["which_one_better"]
        qas = dict(zip(questions, explanations))
        for k, v in qas.items():
            if values.get(k) and values.get(k) == "Both" and not values.get(v):
                raise ValueError(
                    f"Since {k} is true, but you did not answer the question {v} in info-personal sheet"
                )
        return values

    @root_validator
    def checkRelationship(cls, values):
        relationship = values["relationship_to_pa"]
        accompany_to_canada = values["accompany_to_canada"]
        dependant_type = values["dependant_type"]

        if relationship and (accompany_to_canada == None):
            raise ValueError(
                colored(
                    f"In info-personal, since {values['first_name']} {values['last_name']} is {relationship} to principle applicant, but you did not pick if accompany to Canada.",
                    "red",
                )
            )

        if (
            relationship
            and relationship
            in ["Adopted Child", "Child", "Grandchild", "Step-Child", "Step-Grandchild"]
            and dependant_type == None
        ):
            raise ValueError(
                colored(
                    f"In info-personal, since {values['first_name']} {values['last_name']} is {relationship} to principle applicant, but you did not pick dependant type",
                    "red",
                )
            )
        return values


class Education(EducationBase):
    city: Optional[str]
    country: Optional[str]


class M0008Model(CommonModel):
    personal: Personal
    status: Status
    cor: List[COR]
    marriage: Marriage
    address: List[Address]
    phone: List[Phone]
    personid: List[PersonId]
    education: List[Education]
    prcase: PRCase

    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self, excels=None, output_excel_file=None):
        if output_excel_file:
            excels = self.getExcels(["excel/pr.xlsx", "excel/pa.xlsx"])
        else:
            if excels is None and len(excels) == 0:
                raise ValueError(
                    "You must input excel file list as source data for validation"
                )
        super().__init__(excels, output_excel_file, globals())



from datetime import date
from typing import Optional,List
from pydantic import BaseModel,root_validator
from model.common.commonmodel import CommonModel
import os


class General(BaseModel):
    legal_name:str
    operating_name:Optional[str]
    website:Optional[str]
    establish_date:date
    num_pnps:int
    num_pnps_approved:int
    num_pnps_in_process:int
    has_lmia_approved:bool
    when_lmia_approved:Optional[date]
    last_revenue:float
    last_profit:float
    before_last_revenue:float
    before_last_profit:float
    retained_earning:float
    has_jobbank_account:bool
    has_bc_employer_certificate:bool

    
class Employee_list(BaseModel):
    employee: str
    job_title: str
    wage: float
    hours_per_week: float
    immigration_status: str
    employment_start_date: date
    remark: Optional[str]

class Lmi(BaseModel):
    laid_off_in_12: bool
    laid_off_canadians: Optional[int]
    laid_off_tfw: Optional[int]
    laid_off_reason: Optional[str]
    is_work_sharing: bool
    work_sharing_info: Optional[str]
    labour_dispute: bool
    labour_dispute_info: Optional[str]

    @root_validator
    def checkLayoff(cls, values):
        laid_off_in_12 = values.get("laid_off_in_12", None)
        laid_off_canadians = values.get("laid_off_canadians", None)
        laid_off_tfw = values.get("laid_off_tfw", None)
        laid_off_reason = values.get("laid_off_reason", None)
        if laid_off_in_12 and (
            not laid_off_canadians or not laid_off_tfw or not laid_off_reason
        ):
            raise ValueError(
                "Since there is laid of in past 12 months in info lmi sheet,but did not input how many Canadians and/or foreign workers, and/or reason of lay off."
            )
        return values

    @root_validator
    def checkWorkSharing(cls, values):
        is_work_sharing = values.get("is_work_sharing", None)
        work_sharing_info = values.get("work_sharing_info", None)
        if is_work_sharing and not work_sharing_info:
            raise ValueError(
                "Since there is work sharing in info lmi sheet,but did not input the details about it."
            )
        return values
    
class LmiaAssess(CommonModel):
    general:General
    employee_list:List[Employee_list]
    lmi:Lmi
    
    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self,excels=None,output_excel_file=None):
        if output_excel_file:
            excels=self.getExcels(["excel/er.xlsx","excel/lmia.xlsx"])
        else:
            if excels is None and len(excels)==0:
                raise ValueError('You must input excel file list as source data for validation')
        # call parent class for validating
        super().__init__(excels,output_excel_file,globals())




from .context import DATADIR
from model.common.commonmodel import CommonModel
from model.bcpnp.data import General
from model.common.jobposition import PositionBase
from model.common.wordmaker import WordMaker
import os


class Position(PositionBase):
    pass


class CompanyInfoModel(CommonModel):
    general: General
    position: Position
    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self, excels=None, output_excel_file=None):
        if output_excel_file:
            excels = self.getExcels(["excel/er.xlsx"])
        else:
            if excels is None and len(excels) == 0:
                raise ValueError(
                    "You must input excel file list as source data for validation"
                )
        # call parent class for validating
        super().__init__(excels, output_excel_file, globals())


class CompanyInfoDocxAdaptor:
    def __init__(self, employer_training_obj: CompanyInfoModel):
        self.employer_training_obj = employer_training_obj

    def make(self, output_docx):
        template_path = os.path.abspath(
            os.path.join(DATADIR, "word/bcpnp_company_information.docx")
        )
        wm = WordMaker(template_path, self.employer_training_obj, output_docx)
        wm.make()



from typing import List
from model.common.trperson import Family
from model.common.commonmodel import CommonModel

"""
Program model for temporary resident visa. Get and validate info for forms: imm5257, imm0104, imm5257b_1, and imm5645
"""


class M5645Model(CommonModel):
    family: List[Family]

    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self, excels=None, output_excel_file=None):
        if output_excel_file:
            excels = self.getExcels(["excel/pa.xlsx"])
        else:
            if excels is None and len(excels) == 0:
                raise ValueError(
                    "You must input excel file list as source data for validation"
                )
        super().__init__(excels, output_excel_file, globals())



from signal import raise_signal
from xml.dom import ValidationErr
from pydantic import BaseModel
from typing import List, Optional
from datetime import date


class FamilyBase(BaseModel):
    last_name: str
    first_name: str
    native_last_name: Optional[str]
    native_first_name: Optional[str]
    date_of_birth: date
    date_of_death: Optional[date]
    place_of_birth: str
    birth_country: str
    relationship: str


class FamilyMembers:
    def __init__(self, members: List[FamilyBase]):
        self.members = members

    @property
    def spouse(self):
        s = [s for s in self.members if s.relationship == "Spouse"]
        if len(s) == 1:
            return s[0]
        elif len(s) == 0:
            return None
        else:
            raise ValidationErr(
                "Spouse has more than one, please check information in table-family"
            )

    @property
    def siblings(self):
        s = [s for s in self.members if s.relationship in ["Brother", "Sister"]]
        return s

    @property
    def dependants(self):
        s = [s for s in self.members if s.relationship in ["Son", "Daughter"]]
        return s

    @property
    def father(self):
        s = [s for s in self.members if s.relationship == "Father"]
        return s[0] if len(s) == 1 else None

    @property
    def mother(self):
        s = [s for s in self.members if s.relationship == "Mother"]
        return s[0] if len(s) == 1 else None

    @property
    def parents(self):
        s = [s for s in self.members if s.relationship in ["Father", "Mother"]]
        return s



from models.definition import Action
import base64


class WebElement:
    def __init__(self):
        self.delay = 50

    def encode(self, password):
        password_bytes = password.encode("ascii")
        base64_bytes = base64.b64encode(password_bytes)
        return base64_bytes.decode("ascii")

    def inputElement(self, id, value, label="Input", length=100, required=True):
        return {
            "action_type": Action.Input.value,
            "label": label,
            "id": id,
            "value": value,
            "length": length,
            "required": required,
            "delay": self.delay,
        }

    def areatextElement(self, id, value, label="Input", length=100, required=True):
        return {
            "action_type": Action.Areatext.value,
            "label": label,
            "id": id,
            "value": value,
            "length": length,
            "required": required,
            "delay": self.delay,
        }

    def selectElement(self, id, value, label="Select"):
        return {
            "action_type": Action.Select.value,
            "label": label,
            "id": id,
            "value": value,
            "delay": self.delay,
        }

    def checkboxElement(self, id, value, label="Checkbox"):
        return {
            "action_type": Action.Checkbox.value,
            "label": label,
            "id": id,
            "delay": self.delay,
        }

    def radioElement(self, id, value, label="Radio"):
        return {
            "action_type": Action.Radio.value,
            "id": id,
            "label": label,
            "delay": self.delay,
        }

    def buttonElement(self, id, value, label="Button"):
        return {
            "action_type": Action.Button.value,
            "label": "Button",
            "id": id,
            "delay": self.delay,
        }

    def pageElement(self, id, nex_page_tag, actions=[], label="Webpage"):
        return {
            "action_type": Action.WebPage.value,
            "page_name": label,
            "actions": actions,
            "id": id,
            "next_page_tag": nex_page_tag,
        }

    def loginElement(self, account_element_id, account, password_element_id, password):
        return {
            "action_type": Action.Login.value,
            "account": account,
            "password": self.encode(password),
            "account_element_id": account_element_id,
            "password_element_id": password_element_id,
        }

    # security_answers is a dict including questions and answers
    def securityElement(
        self, question_element_id, answer_element_id, security_answers: dict
    ):
        return {
            "action_type": Action.Security.value,
            "question_element_id": question_element_id,
            "answer_element_id": answer_element_id,
            "security_answers": security_answers,
        }

    def dependantSelectElement(self, select1: dict, select2: dict):
        return {
            "action_type": Action.DependantSelect.value,
            "select1": select1,
            "select2": select2,
        }

    def pressKeyElement(self, id, value, label="Press Key"):
        return {
            "action_type": Action.PressKey.value,
            "id": id,
            "key": value,
            "label": label,
            "delay": self.delay,
        }

    def pdfElement(self, wait_for: str):
        return {
            "action_type": Action.Pdf.value,
            "wait_for": wait_for,
            "delay": self.delay,
        }

    def imgElement(self, wait_for: str):
        return {
            "action_type": Action.Image.value,
            "wait_for": wait_for,
            "delay": self.delay,
        }

    def waitElement(self, duration):
        return {"action_type": Action.Wait.value, "duration": duration}

    def uploadElement(self, id, value, label):
        return {
            "action_type": Action.Upload.value,
            "id": id,
            "filename": value,
            "label": "Upload",
            "delay": self.delay,
        }

    def gotoPageElement(self, url, wait_for=None):
        return {"action_type": Action.GotoPage.value, "url": url, "wait_for": wait_for}

    def confirmElement(self, message):
        return {"action_type": Action.Confirm.value, "message": message}



from pydantic import BaseModel, EmailStr, root_validator
from typing import Optional, List
from datetime import date

from model.common.commonmodel import CommonModel


class Family(BaseModel):
    relationship: Optional[str]
    last_name: Optional[str]
    first_name: Optional[str]
    native_last_name: Optional[str]
    native_first_name: Optional[str]
    date_of_birth: Optional[date]
    date_of_death: Optional[date]
    place_of_birth: Optional[str]
    birth_country: Optional[str]
    marital_status: Optional[str]
    email: Optional[EmailStr]
    address: Optional[str]

    @root_validator
    def checkFields(cls, values):
        last_name = values["last_name"]
        first_name = values["first_name"]
        native_last_name = values["native_last_name"]
        native_first_name = values["native_first_name"]
        date_of_birth = values["date_of_birth"]
        place_of_birth = values["place_of_birth"]
        birth_country = values["birth_country"]
        marital_status = values["marital_status"]
        address = values["address"]

        if (
            last_name
            and first_name
            and not all(
                [
                    native_last_name,
                    native_first_name,
                    date_of_birth,
                    place_of_birth,
                    birth_country,
                    marital_status,
                    address,
                ]
            )
        ):
            raise ValueError(
                f"Some fields of {first_name} {last_name} in info-family sheet are not completed, please check"
            )
        return values


class M5406Model(CommonModel):
    family: List[Family]

    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self, excels=None, output_excel_file=None):
        if output_excel_file:
            excels = self.getExcels(["excel/pa.xlsx"])
        else:
            if excels is None and len(excels) == 0:
                raise ValueError(
                    "You must input excel file list as source data for validation"
                )
        super().__init__(excels, output_excel_file, globals())



from datetime import date
from pkg_resources import working_set
from pydantic import BaseModel, root_validator
from typing import Optional, List
from termcolor import colored


class Status(BaseModel):
    current_country: str
    current_country_status: str
    current_workpermit_type: Optional[str]
    has_vr: Optional[bool]
    current_status_start_date: date
    current_status_end_date: date
    other_status_explaination: Optional[str]
    last_entry_date: Optional[date]
    last_entry_place: Optional[date]

    @root_validator
    def checkWorkerType(cls, values):
        current_status = values["current_country_status"]
        workpermit_type = values["current_workpermit_type"]
        if current_status == "Worker" and workpermit_type == None:
            raise ValueError(
                colored(
                    f"Since current stauts is {current_status}, but you did not answer the question followed '工签类别' in info-status sheet",
                    "red",
                )
            )
        return values



from termcolor import colored
from functools import reduce
from typing import List,Optional
from model.common.address import Address
from model.common.phone import Phone
from model.common.trperson import COR, PersonId,Personal,Marriage,Education,Employment,Travel,Family
from model.common.tr import TrCase,Wp,TrBackground
from model.common.commonmodel import CommonModel

"""
Program model for temporary resident visa. Get and validate info for forms: imm5257, imm0104, imm5257b_1, and imm5645
"""
class M1295Model(CommonModel):
    personal:Personal
    marriage:Marriage
    personid:List[PersonId]
    address:List[Address]
    education:List[Education]
    employment:List[Employment]
    travel:List[Travel]
    family:List[Family]
    phone:List[Phone]
    cor:List[COR]
    trcase:TrCase
    wp:Wp
    trbackground:TrBackground
    
    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self,excels=None,output_excel_file=None):
        if output_excel_file:
            excels=self.getExcels(['excel/tr.xlsx','excel/pa.xlsx'])
        else:
            if excels is None and len(excels)==0:
                raise ValueError('You must input excel file list as source data for validation')
        super().__init__(excels,output_excel_file,globals())




from pydantic import BaseModel, validator, EmailStr, root_validator
from datetime import date
from model.common.utils import makeList, Duration
from model.common.person import Person
from model.common.id import ID, IDs
from typing import Optional, List
from model.common.employmentbase import EmploymentBase
from model.common.utils import checkRow


class Employment(EmploymentBase):
    department: Optional[str]
    duties: list
    company_brief: str
    fullname_of_certificate_provider: Optional[str]
    position_of_certificate_provider: Optional[str]
    department_of_certificate_provider: Optional[str]
    phone_of_certificate_provider: Optional[str]
    email_of_certificate_provider: Optional[EmailStr]
    employment_certificate: bool

    _normalize_duties = validator("duties", allow_reuse=True, pre=True)(makeList)

    @root_validator
    def checkCompletion(cls, values):
        all_fields = [
            "job_title",
            "noc_code",
            "weekly_hours",
            "company",
            "city",
            "province",
            "country",
            "department",
            "duties",
            "company_brief",
            "fullname_of_certificate_provider",
            "position_of_certificate_provider",
            "department_of_certificate_provider",
            "phone_of_certificate_provider",
            "email_of_certificate_provider",
            "employment_certificate",
        ]
        required_fields = [
            "job_title",
            "noc_code",
            "weekly_hours",
            "company",
            "city",
            "province",
            "country",
            "department",
            "duties",
            "company_brief",
            "fullname_of_certificate_provider",
            "position_of_certificate_provider",
            "department_of_certificate_provider",
            "phone_of_certificate_provider",
            "email_of_certificate_provider",
            "employment_certificate",
        ]

        checkRow(values, all_fields, required_fields)

        return values
